! EVAZOR -- A side-scrolling ASCII space game
! By Nils M Holm, 2024
! In the public domain / under the 0BSD license

use t3x: t;
use host;
use console: con;
use rng;

version() return "1.5";

const	COLS	= 80,
	LINES	= 23;

const	SPEEDUP	= 100;

const	NMISSILES = 6;

var	Smoothness;

var	Sx, Sy;

var	Types::COLS;
var	Objects::COLS, Top::COLS, Bottom::COLS;
var	Topseed, Botseed;

var	Speed, Tick, Pos, Objtime;

var	Shields, Missiles, Credits;
var	Flash, Flashtype;
var	Fired::NMISSILES, Mx::NMISSILES, My::NMISSILES;
var	Level;

var	Upkey, Downkey, Firekey, Fire2key;

var	Mdelay, Slowdown;

struct	CONF =	CNF_LDELAY,
		CNF_HDELAY,
		CNF_UPKEY,
		CNF_DOWNKEY,
		CNF_FIREKEY,
		CNF_FIRE2KEY,
		CNF_FXSLOWDN;

struct	SCORE =	SC_LSCORE,
		SC_HSCORE,
		SC_LEVEL,
		SC_RESERVED,
		SC_NAME;

var	Scores::192;

var	Csh1, Csh0, Cshsep;
var	CUL, CUR, CLL, CLR, CVL, CHL;
var	Cmap, Cship;
var	Cupwall, Clowall;
var	Cdebris, Ccredit, Cshield, Cmissile, Ctypes::3;

box(x, y, dx, dy) do var i, j;
	con.move(x, y);
	con.wrch(CUL);
	for (j=x+2, dx) con.wrch(CHL);
	con.wrch(CUR);
	for (i=y+1, dy-1) do
		con.move(x, i);
		con.wrch(CVL);
		for (j=x+2, dx) con.wrch('\s');
		con.wrch(CVL);
	end
	con.move(x, dy-1);
	con.wrch(CLL);
	for (j=x+2, dx) con.wrch(CHL);
	con.wrch(CLR);
end

drawship() do
	con.move(Sx, Sy);   con.writes("|\\=- ");
	con.move(Sx, Sy+1); con.writes(">=o==");
	con.move(Sx, Sy+2); con.writes("|/=- ");
end

shieldflash() do
	con.move(Sx, Sy);   con.writes("&&&& ");
	con.move(Sx, Sy+1); con.writes("&&&&&");
	con.move(Sx, Sy+2); con.writes("&&&& ");
end

collectflash() do
	con.move(Sx, Sy);   con.writes("%%%% ");
	con.move(Sx, Sy+1); con.writes("%%%%%");
	con.move(Sx, Sy+2); con.writes("%%%% ");
end

drawflash() do
	ie (Flashtype = '&')
		shieldflash();
	else
		collectflash();
end

var	Ntoa_buf::10;

ntoa(n, k) do var i, j;
	i := 9;
	Ntoa_buf::9 := 0;
	while (n \/ i = 9) do
		i := i - 1;
		Ntoa_buf::i := n mod 10 + '0';
		n := n ./ 10;
	end
	for (j=9-i, k) do
		i := i - 1;
		Ntoa_buf::i := '0';
	end
	return @Ntoa_buf::i;
end

status() do var i, j;
	con.move(0, LINES);
	con.clreol();
	con.writes("[[ SCORE ");
	con.writes(ntoa(Credits, 5));
	ie (HOST.DOS)
		con.writes(packed [ " ]",222,222,222,222,"[ ", 0 ]);
	else
		con.writes(" ]||||[ ");
	for (i=0, 38) con.wrch(Cmap);
	ie (HOST.DOS)
		con.writes(packed [ " ]",221,221,221,221,"[ LEVEL ", 0 ]);
	else
		con.writes(" ]||||[ LEVEL ");
	con.writes(ntoa(Level, 2));
	con.writes(" ]]");
	con.move(Pos*2+22, LINES);
	con.writes(Cship);
	con.move(0, 1);
	if (Missiles) con.standout();
	con.writes(ntoa(Missiles, 2));
	con.normal();
	for (i=0, Shields) do
		for (j=1, 3) do
			con.move(0, LINES-i*3-j);
			con.writes(Csh1);
		end
		con.move(0, LINES-i*3-3);
		con.writes(Cshsep);
	end
	for (i=Shields, 7) do
		for (j=1, 3) do
			con.move(0, LINES-i*3-j);
			con.writes(Csh0);
		end
		con.move(0, LINES-i*3-3);
		con.writes(Cshsep);
	end
end

scrollset(coords, obj) do var i;
	for (i = 2, COLS) do
		if (coords::i \= 255 /\ (obj \/ Types::i)) do
			ie (i > 2) do
				con.move(i-1, coords::i);
				con.wrch(obj-> obj: Types::i);
				con.wrch('\s');
				coords::(i-1) := coords::i;
				if (\obj) Types::(i-1) := Types::i;
				coords::i := 255;
			end
			else do
				con.move(2, coords::i);
				con.wrch('\s');
			end
		end
	end
end

scroll() do var now;
	scrollset(Objects, 0);
	Tick := Tick + 1;
	if (Tick mod Speed = 0 /\ Pos < 17) do
		Objects::(COLS-1) :=
			rng.rand(Botseed - Topseed - 2) + Topseed + 1;
		Types::(COLS-1) := Cdebris;
		ie (rng.rand(100) >= 90-(Objtime>>2)) do
			Types::(COLS-1) := Ctypes::rng.rand(3);
			Objtime := 0;
		end
		else do
			Objtime := Objtime + 1;
		end
		con.move(COLS-1, Objects::(COLS-1));
		con.wrch(Types::(COLS-1));
	end
	scrollset(Top, Cupwall);
	if (Pos < 18) Top::(COLS-1) := Topseed;
	con.move(COLS-1, Top::(COLS-1));
	con.wrch(Cupwall);
	scrollset(Bottom, Clowall);
	if (Pos < 18) Bottom::(COLS-1) := Botseed;
	con.move(COLS-1, Bottom::(COLS-1));
	con.wrch(Clowall);
	now := Tick mod Smoothness = 0;
	ie (now /\ Pos >= 17) do
		Topseed := Topseed - 1;
		if (Topseed < 0) Topseed := 0;
		Botseed := Botseed + 1;
		if (Botseed > LINES-1) Botseed := LINES-1;
	end
	else if (now) do
		Topseed := Topseed + rng.rand(3)-1;
		if (Topseed < 0) Topseed := 0;
		if (Topseed > 3+Level) Topseed := 3+Level;
		Botseed := Botseed + rng.rand(3)-1;
		if (Botseed - Topseed < 8) do
			Botseed := Botseed + 1;
			Topseed := Topseed - 1;
		end
		if (Botseed > LINES-1) Botseed := LINES-1;
		if (Botseed < LINES-3-Level) Botseed := LINES-3-Level;
	end
end

collision_in(coords) do var i, j;
	for (i=Sx, Sx+4)
		for (j=Sy, Sy+3)
			if (coords::i = j) return i;
	if (Coords::(Sx+4) = Sy+1) return Sx+4;
end

collision() do var x, obj;
	x := collision_in(Objects);
	if (x) do
		obj := Types::x;
		Types::x := 0;
		drawship();
		return obj;
	end
	x := collision_in(Top);     if (x) return %1;
	x := collision_in(Bottom);  if (x) return %1;
end

shipup() do
	if (Sy = 0) return;
	con.move(Sx,Sy+2);
	con.writes("\s\s\s\s\s");
	Sy := Sy - 1;
	drawship();
end

shipdown() do
	if (Sy >= LINES-3) return;
	con.move(Sx,Sy);
	con.writes("\s\s\s\s\s");
	Sy := Sy + 1;
	drawship();
end

disp(x, y, c) do
	if (x >= 0 /\ x < COLS /\ y >= 0 /\ y < LINES) do
		con.move(x, y);
		con.wrch(c);
	end
end

parkcsr() con.move(COLS-1, LINES);

startip(x, y, d, c) do
	if (d < 8) do
		disp(x+d, y+d, c);
		disp(x-d, y+d, c);
		disp(x+d, y-d, c);
		disp(x-d, y-d, c);
	end
	disp(x,   y+d, c);
	disp(x,   y-d, c);
	disp(x+d, y,   c);
	disp(x-d, y,   c);
	parkcsr();
	con.wait(100 * Slowdown);
end

star(x, y, c) do var i;
	for (i=0, 10) startip(x, y, i, c);
end

boom() do
	star(Sx+2, Sy+1, '@');
	star(Sx+2, Sy+1, '\s');
end

collect(n) do
	Flash := 4;
	Flashtype := '%';
	Credits := Credits + n;
	status();
end

shieldsup() do
	Flash := 4;
	Flashtype := '%';
	if (Shields = 7) return collect(200);
	Shields := Shields + 1;
	status();
end

firepower() do
	Flash := 4;
	Flashtype := '%';
	Missiles := Missiles + 5;
	if (Missiles > 99) do
		Missiles := 99;
		return collect(150);
	end
	status();
end

fire() do var i;
	if (Missiles = 0) return;
	for (i=0, NMISSILES)
		if (\Fired::i) leave;
	if (i = NMISSILES) return;
	Mx::i := Sx + 4;
	My::i := Sy + 1;
	Fired::i := %1;
	Missiles := Missiles - 1;
	status();
end

fire2() do var i, j;
	if (Missiles < 2) return;
	j := %1;
	for (i=0, NMISSILES)
		if (\Fired::i)
			ie (j = %1)
				j := i;
			else
				leave;
	if (i = NMISSILES) return;
	Mx::i := Sx + 4;
	My::i := Sy;
	Mx::j := Sx + 4;
	My::j := Sy + 2;
	Fired::i := %1;
	Fired::j := %1;
	Missiles := Missiles - 2;
	status();
end

movemsl(n) do var mx0, mx1, my0;
	mx0 := Mx::n+1;
	mx1 := mx0 + 1;
	my0 := My::n;
	if (	Top::mx0 = my0 \/ Bottom::mx0 = my0 \/
		Top::mx1 = my0 \/ Bottom::mx1 = my0 \/
		mx0 >= COLS-1)
	do
		con.move(mx0, my0);
		con.wrch('\s');
		Fired::n := 0;
		return;
	end
	if (Objects::mx0 = my0) do
		Objects::mx0 := 255;
		Fired::n := 0;
		con.move(mx0, my0);
		con.writes("\s\s");
		return;
	end
	if (Objects::mx1 = my0) do
		Objects::mx1 := 255;
		Fired::n := 0;
		con.move(mx0, my0);
		con.writes("\s\s");
		return;
	end
	con.move(mx0, my0);
	con.writes(" -");
	Mx::n := Mx::n + 1;
end

movemsls() do var i;
	for (i=0, NMISSILES)
		if (Fired::i)
			movemsl(i);
end

length(s) return t.memscan(s, 0, 32767);

message(s) do var i, k;
	con.move(0, LINES);
	con.standout();
	for (i=0, 39) con.wrch('>');
	for (i=0, 40) con.wrch('<');
	k := length(s);
	con.move(39-k/2, LINES);
	con.wrch('\s');
	con.writes(s);
	con.wrch('\s');
	con.normal();
	parkcsr();
	k := con.getkey();
	status();
	return k;
end

quit() do var k;
	k := message("PAUSED -- PRESS 'Y' TO QUIT");
	return k = 'y' \/ k = 'Y';
end

setup() do
	t.memfill(Objects, 255, COLS);
	t.memfill(Top, 255, COLS);
	t.memfill(Bottom, 255, COLS);
	t.memfill(Fired, 0, NMISSILES);
	Sx := 15;
	Sy := 10;
	Speed := 10;
	Tick := 0;
	Topseed := 0;
	Botseed := LINES-1;
	Flash := 0;
	Pos := 0;
	Objtime := 0;
end

playlev() do var k, i, obj, over;
	over := "GAME OVER -- PRESS ESC";
	setup();
	con.clrscr();
	drawship();
	status();
	while (%1) do
		if (Flash) do
			ie (Flash & 1)
				drawship();
			else
				drawflash();
			Flash := Flash - 1;
		end
		scroll();
		if (Fired) movemsls();
		parkcsr();
		con.wait(100);
		k := con.pollkey();
		     ie (k = 'q') if (quit()) return 'q';
		else ie (k = Upkey) shipup();
		else ie (k = Downkey) shipdown();
		else ie (k = Firekey) fire();
		else ie (k = Fire2key) fire2();
		else ;
		rng.reseed(k);
		obj := collision();
		     ie (obj = Ccredit) collect(100);
		else ie (obj = Cshield) shieldsup();
		else ie (obj = Cmissile) firepower();
		else if (obj) do
			if (\Shields) do
				boom();
				while (message(over) \= '\e')
					;
				return %1;
			end
			Flash := 6;
			Flashtype := '&';
			Shields := Shields - 1;
			status();
		end
		if (Tick mod SPEEDUP = 0) do
			if (Speed > 2) Speed := Speed - 1;
			Pos := Pos + 1;
			Credits := Credits + 100;
			status();
			if (Pos = 18) leave;
		end
	end
	for (i=0, COLS-1) do
		con.pollkey();
		if (i & 1) do
			if (Sy < 10) shipdown();
			if (Sy > 10) shipup();
		end
		scrollset(Top, Cupwall);
		scrollset(Bottom, Clowall);
		parkcsr();
		con.wait(100);
	end
end

loadconf() do var i, fd, k;
	t.memfill(Scores, '.', 192);
	for (i=16, 192, 16) do
		Scores::(i+SC_LSCORE) := 0;
		Scores::(i+SC_HSCORE) := 0;
		Scores::(i+SC_LEVEL) := 0;
		Scores::(i+SC_RESERVED) := 0;
	end
	fd := t.open("evazor.cnf", T3X.OREAD);
	if (fd < 0) return %1;
	k := t.read(fd, Scores, 192);
	t.close(fd);
	if (k \= 192) return %1;
	Mdelay := Scores::CNF_HDELAY << 8 | Scores::CNF_LDELAY;
	Upkey := Scores::CNF_UPKEY;
	Downkey := Scores::CNF_DOWNKEY;
	Firekey := Scores::CNF_FIREKEY;
	Fire2key := Scores::CNF_FIRE2KEY;
	Slowdown := Scores::CNF_FXSLOWDN;
	con.calibrate(Mdelay);
end

saveconf() do var fd, k;
	Scores::CNF_LDELAY := Mdelay;
	Scores::CNF_HDELAY := Mdelay >> 8;
	Scores::CNF_UPKEY := Upkey;
	Scores::CNF_DOWNKEY := Downkey;
	Scores::CNF_FIREKEY := Firekey;
	Scores::CNF_FIRE2KEY := Fire2key;
	Scores::CNF_FXSLOWDN := Slowdown;
	fd := t.open("evazor.cnf", T3X.OWRITE);
	t.write(fd, Scores, 192);
	t.close(fd);
end

text(x, y, b) do var i;
	i := 0;
	while (b[i]) do
		con.move(x, y+i);
		con.writes(b[i]);
		i := i + 1;
	end
end

prompt(x, y, s) do var k, n;
	n := 0;
	t.memfill(s, '.', 12);
	con.csron();
	while (%1) do
		con.move(x+n, y);
		k := con.getkey();
		ie (k = '\b' /\ n > 0) do
			s::n := '.';
			n := n - 1;
			con.move(x+n, y);
			con.wrch('_');
		end
		else ie (k = '\r') do
			con.csroff();
			return;
		end
		else if ('\s' <= k /\ k <= '~' /\ n < 11) do
			s::n := k;
			n := n + 1;
			con.wrch(k);
		end
	end
end

highscore() do var i, sc;
	for (i=1, 12) do
		sc := Scores::(16*i+SC_HSCORE) << 8
		    | Scores::(16*i+SC_LSCORE);
		if (Credits .> sc) leave;
	end
	if (i < 11) do
		i := i*16;
		text(23, 9, [   "                                     ",
				"  >>>>>>>>>> HIGH SCORE <<<<<<<<<<<  ",
				"  +-------------------------------+  ",
				"  |                               |  ",
				"  +-------------------------------+  ",
				"                                     ", 0]);
		box(25, 11, 58, 14);
		text(27, 12, ["ENTER YOUR NAME: ____________", 0]);
		if (i < 176)
			t.memcopy(@Scores::(i+16), @Scores::i, 176-i);
		Scores::(i+SC_LSCORE) := Credits;
		Scores::(i+SC_HSCORE) := Credits >> 8;
		Scores::(i+SC_LEVEL) := Level;
		prompt(44, 12, @Scores::(i+SC_NAME));
	end
	saveconf();
end

play() do var k;
	Shields := 1;
	Missiles := 5;
	Credits := 0;
	Smoothness := 10;
	Level := 1;
	con.clrscr();
	setup();
	status();
	drawship();
	while (%1) do
		message("GET READY");
		k := playlev();
		if (k) leave;
		Level := Level + 1;
		Smoothness := Smoothness >> 1;
		if (Smoothness = 0) Smoothness := 10;
	end
	highscore();
end

title() do var evazor, i, j, k, k2, x, y;
	evazor := [ " ____  _  __ _   ____  ____  ____ ",
		    "| ___)| |/ // \\ (_   )/    \\| _  \\",
		    "| __) |   // _ \\  / /_| () ||    /",
		    "|____)|__//_/ \\_\\(____)____/|_|\\_\\", 0 ];
	k := length(evazor[0]);
	k2 := k / 2;
	x := 40;
	y := 2;
	con.clrscr();
	for (i=0, k+1) do
		con.move(x-k2, y);
		for (j=0, k-i-1) con.wrch('\s');
		for (j=0, i) con.wrch(evazor[0]::j);
		con.move(x-k2, y+1);
		con.writes(@evazor[1]::k-i);
		con.move(x-k2, y+2);
		for (j=0, k-i-1) con.wrch('\s');
		for (j=0, i) con.wrch(evazor[2]::j);
		con.move(x-k2, y+3);
		con.writes(@evazor[3]::k-i);
		parkcsr();
		con.wait(20);
		con.pollkey();
	end
	for (i=0, 5) do
		con.move(x+2+k2+i*4, y+1); con.writes("\\\\");
		con.move(x+3+k2+i*4, y+2); con.writes("))");
		con.move(x+2+k2+i*4, y+3); con.writes("//");
		con.move(x-4-k2-i*4, y+1); con.writes("//");
		con.move(x-5-k2-i*4, y+2); con.writes("((");
		con.move(x-4-k2-i*4, y+3); con.writes("\\\\");
		parkcsr();
		con.wait(50);
		con.refresh();
	end
end

putc(x, y, c) do
	con.move(x, y);
	con.wrch(c);
end

puts(x, y, s) do
	con.move(x, y);
	con.writes(s);
end

hline(x, y, dx, c) do var i;
	for (i=x, dx) putc(i, y, c);
end

setspeed(k)
	if ('1' <= k /\ k <= '5')
		Slowdown := k - '0';

fxdelay() do var k, c, d, sd;
	con.clrscr();
	Sx := COLS-25;
	Sy := LINES/2;
	text(10, 5, [
		"VISUAL EFFECT SPEED",
		"",
		"[1] = Fast",
		"[2] = Medium",
		"[3] = Slow",
		"",
		"[q]  = quit",
		"[CR] = save & exit", 0 ]);
	d := 0;
	c := '@';
	sd := Slowdown;
	while (%1) do
		startip(Sx, Sy, d, c);
		k := con.pollkey();
		setspeed(k);
		if (k = 'q') do Slowdown := sd; return; end
		if (k = '\r') return;
		d := d + 1;
		if (d = 10) do
			d := 0;
			c := c = '@'-> '\s': '@';
		end
	end
end

calibrate() do var k, i, c, old;
	con.clrscr();
	c := '#';
	hline(15, 8, 65, '-');
	putc(25, 8, '|'); puts(25, 9, "1s");
	putc(35, 8, '|'); puts(35, 9, "2s");
	putc(45, 8, '|'); puts(45, 9, "3s");
	putc(55, 8, '|'); puts(55, 9, "4s");
	putc(65, 8, '|'); puts(65, 9, "5s");
	i := 0;
	old := Mdelay;
	puts(28,  2, "TIMER LOOP CALIBRATION");
	puts(15,  5, "This should be about 5 seconds:");
	puts(15, 13, "[+]  = Slower (+100)    [-] = Faster (-100)");
	puts(15, 14, "[>]  = Slower  (+10)    [<] = Faster  (-10)");
	puts(15, 15, "[)]  = Slower   (+1)    [(] = Faster   (-1)");
	puts(15, 16, "[CR] = Save + exit      [q] = Quit (don't save)");
	puts(31, 19, "TOO SLOW? PRESS [s]");
	while (%1) do
		if (i = 0) c := c = '#'-> ':': '#';
		puts(15, 11, "Delay: ");
		con.clreol();
		con.writes(ntoa(Mdelay, 0));
		if (Mdelay < 10)
			con.writes(" [Your machine is too slow!]");
		putc(15+i, 7, c);
		con.wrch(c);
		con.calibrate(Mdelay);
		con.wait(100);
		i := (i + 1) mod 50;
		k := con.pollkey();
		     ie (k = '+') Mdelay := Mdelay + 100;
		else ie (k = '-') Mdelay := Mdelay - 100;
		else ie (k = '>') Mdelay := Mdelay + 10;
		else ie (k = '<') Mdelay := Mdelay - 10;
		else ie (k = ')') Mdelay := Mdelay + 1;
		else ie (k = '(') Mdelay := Mdelay - 1;
		else ie (k = 's') Mdelay := 100;
		else ie (k = '\r' \/ k = '\n') leave;
		else if (k = 'q') do Mdelay := old; leave; end
		if (Mdelay < 0) Mdelay := 0;
	end
	con.clrscr();
	con.calibrate(Mdelay);
	fxdelay();
	saveconf();
end

key(y, ks, k) do var nk;
	while (%1) do
		con.move(57, y);
		nk := con.getkey();
		if ('\s' <= nk /\ nk <= '~' /\ nk \= 'q') leave;
	end
	con.move(57, y);
	con.wrch(nk);
	return nk;
end

confkeys() do
	con.csron();
	Upkey := key(8, "   Up", Upkey);
	Downkey := key(9, " Down", Downkey);
	Firekey := key(10, " Fire", Firekey);
	Fire2key := key(11, "Fire2", Firekey);
	con.csroff();
	saveconf();
end

instructions() do var i;
	setup();
	con.clrscr();
	Level := 1;
	Missiles := 10;
	Shields := 3;
	Credits := 900;
	Smoothness := 20;
	status();
	for (i=0, COLS-2) scroll();
	drawship();
	puts(Sx-2, Sy+4, "Your Ship");
	putc(3, 2, '\\');
	puts(4, 3, "Missiles Left");
	puts(3, 17, "-- Shields Left");
	puts(22, LINES-1, ".-----------------^------------------.");
	putc(40, LINES-2, '|');
	putc(40, LINES-3, '|');
	putc(40, LINES-4, '|');
	puts(34, LINES-5, "Level Progress");
	box(30, 7, 70, 14);
	text(33, 8, [	" #  Rock wall - instant death      ",
			" @  Space debris - evade this      ",
			" %  Credits - collect those        ",
			" *  Missiles - good to have        ",
			" &  Shields - your life ensurance  ",
			0 ]);
	con.move(34,  8); con.wrch(Cupwall);
	con.move(34,  9); con.wrch(Cdebris);
	con.move(34, 10); con.wrch(Ccredit);
	con.move(34, 11); con.wrch(Cmissile);
	con.move(34, 12); con.wrch(Cshield);
	text(39, 14, ["Press any key when done", 0]);
	parkcsr();
	con.getkey();
end

showsc(x, y, n) do var i, b;
	b := ntoa(n, 2);
	if (b::0 = '0') b::0 := '\s';
	con.move(x, y);
	con.writes(b);
	con.writes(". ");
	n := n*16;
	con.writes(ntoa(Scores::(n+SC_HSCORE) << 8
	              | Scores::(n+SC_LSCORE), 5));
	con.wrch('\s');
	for (i=SC_NAME, 16) con.wrch(Scores::(n+i));
end

showscores() do var i;
	puts(32, 14, "THE GREATEST EVAZORS");
	showsc(31, 16, 1);
	for (i=2, 7) showsc(17, 16+i, i);
	for (i=7, 12) showsc(42, 11+i, i);
end

menu() do var i, k, main, keys, cred;
	main := [ "P - Play",
		  "I - Instructions",
		  "K - Keys",
		  "C - Calibrate",
		  "E - Exit", 0 ];
	keys := [ "Controls:",
		  "[ ] Up",
		  "[ ] Down",
		  "[ ] Fire",
		  "[ ] Fire2",
		  "[q] Pause/quit", 0 ];
	cred := [ "By /\\  |\\  /|   \\  ",
		  "  /  \\ | \\/ |____\\ ",
		  " /    \\|    |     \\",
		  "                2024", 0 ];
	con.clrscr();
	title();
	text(35, 7, main);
	text( 9, 8, cred);
	text(56, 7, keys);
	puts(COLS-6, 6, version());
	showscores();
	while (%1) do
		putc(57,  8, Upkey);
		putc(57,  9, Downkey);
		putc(57, 10, Firekey);
		putc(57, 11, Fire2key);
		parkcsr();
		k := con.getkey();
		     ie (k = 'p') play();
		else ie (k = 'i') instructions();
		else ie (k = 'c') calibrate();
		else ie (k = 'k') do confkeys(); loop; end
		else ie (k = 'e') leave;
		else loop;
		con.clrscr();
		title();
		text(35, 7, main);
		text( 9, 8, cred);
		text(56, 7, keys);
		showscores();
	end
	con.clrscr();
	for (i=0, 12) do
		box(35, i, 46, i+3);
		text(37, i+1, ["GOODBYE", 0]);
		if (i > 0) text(35, i-1, ["           ", 0]);
		parkcsr();
		con.wait(50);
	end
	con.move(0, LINES);
end

init() do
	ie (HOST.DOS) do
		Csh1 := packed [ 178, 178, 0 ];
		Csh0 := packed [ 176, 176, 0 ];
		Cshsep := packed [ 205, 205, 0 ];
		Cmap := 250;
		Cship := packed [ 16, 16, 0 ];
		Cupwall := 220;
		Clowall := 223;
		Cdebris := 254;
		Ccredit := 157;
		Cshield := 127;
		Cmissile := 240;
		CUL := 218;
		CUR := 191;
		CLL := 192;
		CLR := 217;
		CVL := 179;
		CHL := 196;
	end
	else do
		Csh1 := "&&";
		Csh0 := "..";
		Cshsep := "==";
		Cmap := ':';
		CShip := ">>";
		Cupwall := '#';
		Clowall := '#';
		Cdebris := '@';
		Ccredit := '%';
		Cshield := '&';
		Cmissile := '*';
		CUL := '+';
		CUR := '+';
		CLL := '+';
		CLR := '+';
		CVL := '|';
		CHL := '-';
	end
	Ctypes::0 := Ccredit;
	Ctypes::1 := Cshield;
	Ctypes::2 := Cmissile;
	con.setup([24,
		   "\e[H\e[J",
		   "\e[K",
		   "\e[%yd1;%xd1H",
		   "\eM",
		   "\e[?25l",
		   "\e[?25h",
		   "\e[7m",
		   "\e[m"]);
	con.csroff();
	Mdelay := 1000;
	Upkey := 'k';
	Downkey := 'j';
	Firekey := 'a';
	Fire2key := 's';
	Slowdown := 1;
	if (loadconf()) calibrate();
	if (Slowdown > 3) Slowdown := 1;
end

fini() do
	con.csron();
	con.move(0, LINES);
	con.shutdown();
end

do
	init();
	menu();
	fini();
end
